# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
import django.core.files.storage


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='UploadedFile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(default=django.utils.timezone.now)),
                ('uploaded_file', models.FileField(storage=django.core.files.storage.FileSystemStorage(), max_length=255, upload_to=b'temp_uploads')),
                ('original_filename', models.CharField(max_length=255)),
                ('field_name', models.CharField(max_length=255, null=True, blank=True)),
                ('file_id', models.CharField(max_length=40)),
                ('form_id', models.CharField(max_length=40)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



from django.db import models

try:
    from django.urls import reverse as url_reverse
except ImportError:
    from django.core.urlresolvers import reverse as url_reverse


class ModelManager(models.Manager):
    def try_get(self, **kwargs):
        qs = self.get_queryset().filter(**kwargs)
        if qs.exists():
            return qs.get()
        else:
            return None


def get_list(v):
    if isinstance(v, list):
        return v
    else:
        return [v]



from django.conf.urls import url

from . import views


urlpatterns = (
    url(r'^handle_upload$', views.handle_upload, name='file_form_handle_upload'),
    url(r'^handle_delete$', views.DeleteFile.as_view(), name='file_form_handle_delete_no_args'),
    url(r'^handle_delete/(?P<file_id>[\w-]+)$', views.DeleteFile.as_view(), name='file_form_handle_delete'),
)



from django.core.exceptions import PermissionDenied
from django.http import HttpResponse, HttpResponseNotFound
from django.views import generic

from .models import UploadedFile
from .uploader import FileFormUploader
from . import conf


handle_upload = FileFormUploader()


class DeleteFile(generic.View):
    def post(self, request, file_id=None):
        file_id = file_id or request.POST.get('qquuid')

        return self.delete_file(request, file_id)

    def delete(self, request, file_id):
        return self.delete_file(request, file_id)

    def delete_file(self, request, file_id):
        if conf.MUST_LOGIN and not request.user.is_authenticated():
            raise PermissionDenied()

        uploaded_file = UploadedFile.objects.try_get(file_id=file_id)

        if uploaded_file:
            uploaded_file.delete()
            return HttpResponse("ok")
        else:
            return HttpResponseNotFound()



from __future__ import print_function

from django.core.management.base import BaseCommand

from django_file_form.models import UploadedFile


class Command(BaseCommand):
    def handle(self, *args, **options):
        deleted_files = UploadedFile.objects.delete_unused_files()

        verbosity = options.get('verbosity')
        if verbosity:
            if not deleted_files:
                print('No files deleted')
            else:
                print('Deleted files: {0!s}'.format(', '.join(deleted_files)))



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-16 14:01
from __future__ import unicode_literals

import django.core.files.storage
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_file_form', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='uploadedfile',
            name='uploaded_file',
            field=models.FileField(max_length=255, storage=django.core.files.storage.FileSystemStorage(), upload_to='temp_uploads'),
        ),
    ]



import json

from django.forms import ClearableFileInput
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe


class UploadWidgetMixin(ClearableFileInput):
    def render(self, name, value, attrs=None):
        uploaded_files = []
        existing_files = []

        if value:
            if isinstance(value, list):
                values = value
            else:
                values = [value]

            for file_info in values:
                if getattr(file_info, 'existing', False):
                    existing_files.append(file_info.get_values())
                elif hasattr(file_info, 'file_id'):
                    uploaded_files.append(file_info.get_values())
                else:
                    uploaded_files.append(dict(name=file_info.name))

        return mark_safe(
            render_to_string(
                'django_file_form/upload_widget.html',
                dict(
                    input=super(UploadWidgetMixin, self).render(name, value, attrs),
                    uploaded_files=json.dumps(uploaded_files),
                    existing_files=existing_files
                )
            )
        )


class UploadWidget(UploadWidgetMixin, ClearableFileInput):
    pass



from pathlib import Path

from six import python_2_unicode_compatible, text_type

from django.core.files.storage import FileSystemStorage
from django.conf import settings
from django.db import models
from django.core.files import File
from django.utils import timezone

from .util import ModelManager


class UploadedFileManager(ModelManager):
    def delete_unused_files(self, delete=True, now=None):
        deleted_files = []

        for t in self.get_queryset():
            if t.must_be_deleted(now):
                if delete:
                    t.delete()

                deleted_files.append(Path(t.uploaded_file.name).name)

        temp_path = Path(settings.MEDIA_ROOT).joinpath('temp_uploads')

        for f in temp_path.iterdir():
            basename = f.name

            if not self.get_for_file(basename):
                if delete:
                    f.unlink()

                deleted_files.append(basename)

        return deleted_files

    def get_for_file(self, filename):
        return self.try_get(
            uploaded_file='temp_uploads/{0!s}'.format(filename)
        )


@python_2_unicode_compatible
class UploadedFile(models.Model):
    fs = FileSystemStorage()

    created = models.DateTimeField(default=timezone.now)
    uploaded_file = models.FileField(max_length=255, upload_to='temp_uploads', storage=fs)
    original_filename = models.CharField(max_length=255)
    field_name = models.CharField(max_length=255, null=True, blank=True)
    file_id = models.CharField(max_length=40)
    form_id = models.CharField(max_length=40)

    objects = UploadedFileManager()

    def __str__(self):
        return text_type(self.original_filename or '')

    def delete(self, using=None):
        super(UploadedFile, self).delete(using)

        if self.uploaded_file:
            path = self.get_path()

            if path.exists():
                path.unlink()

    def must_be_deleted(self, now=None):
        now = now or timezone.now()

        return (now - self.created).days >= 1

    def get_path(self):
        if self.uploaded_file:
            return Path(self.uploaded_file.path)
        else:
            return None

    def get_uploaded_file(self):
        return UploadedFileWithId(
            self.uploaded_file,
            self.original_filename,
            self.file_id
        )


class UploadedFileWithId(File):
    def __init__(self, _file, name, file_id):
        super(UploadedFileWithId, self).__init__(_file, name)

        self.file_id = file_id

    def get_values(self):
        return dict(id=self.file_id, name=self.name)



from django.forms import FileField
from django.core import validators

from .widgets import UploadWidget
from .models import UploadedFile
from .util import get_list


class UploadedFileField(FileField):
    widget = UploadWidget

    def get_file_data(self, field_name, form_id):
        qs = self._get_file_qs(field_name, form_id)

        if qs.exists():
            return qs.latest('created').get_uploaded_file()
        else:
            return None

    def delete_file_data(self, field_name, form_id):
        qs = self._get_file_qs(field_name, form_id)

        for f in qs:
            f.delete()

    def _get_file_qs(self, field_name, form_id):
        return UploadedFile.objects.filter(
            form_id=form_id,
            field_name=field_name
        )


class MultipleUploadedFileField(UploadedFileField):
    def widget_attrs(self, widget):
        attrs = super(MultipleUploadedFileField, self).widget_attrs(widget)

        attrs['multiple'] = 'multiple'
        return attrs

    def get_file_data(self, field_name, form_id):
        qs = self._get_file_qs(field_name, form_id)

        return [
            f.get_uploaded_file() for f in qs
        ]

    def to_python(self, data):
        if data in validators.EMPTY_VALUES:
            return None
        elif isinstance(data, list):
            return [
                super(MultipleUploadedFileField, self).to_python(f)
                for f in data
            ]
        else:
            return [data]

    def bound_data(self, data, initial):
        result = []

        if initial:
            result += get_list(initial)

        if data:
            result += get_list(data)

        return result


import uuid

import six

from django.forms import CharField, HiddenInput

from .util import url_reverse as reverse

# UploadedFileField and MultipleUploadedFileField must be in this module because they are in the api
from .fields import UploadedFileField, MultipleUploadedFileField


class FileFormMixin(object):
    def __init__(self, *args, **kwargs):
        super(FileFormMixin, self).__init__(*args, **kwargs)

        self.add_hidden_field('form_id', uuid.uuid4)
        self.add_hidden_field('upload_url', self.get_upload_url())
        self.add_hidden_field('delete_url', reverse('file_form_handle_delete_no_args'))

    def add_hidden_field(self, name, initial):
        self.fields[name] = CharField(widget=HiddenInput, initial=initial, required=False)

    def get_upload_url(self):
        return reverse('file_form_handle_upload')

    def full_clean(self):
        if not self.is_bound:
            # Form is unbound; just call super
            super(FileFormMixin, self).full_clean()
        else:
            # Update file data of the form
            self.update_files_data()

            # Call super
            super(FileFormMixin, self).full_clean()

    def update_files_data(self):
        form_id = self.data.get('form_id')

        if form_id:
            for field_name, field in six.iteritems(self.fields):
                if hasattr(field, 'get_file_data'):
                    file_data = field.get_file_data(field_name, form_id)

                    if file_data:
                        self.files[field_name] = file_data

    def delete_temporary_files(self):
        form_id = self.data.get('form_id')

        if form_id:
            for field_name, field in six.iteritems(self.fields):
                if hasattr(field, 'delete_file_data'):
                    field.delete_file_data(field_name, form_id)

    def add_existing_file(self, field_name, filename, delete_url=None, view_url=None):
        self.initial.setdefault(field_name, [])

        existing_file = ExistingFile(name=filename, delete_url=delete_url, view_url=view_url)

        self.initial[field_name].append(existing_file)


class ExistingFile(object):
    def __init__(self, name, delete_url=None, view_url=None):
        self.name = name
        self.delete_url = delete_url
        self.view_url = view_url
        self.existing = True

    def get_values(self):
        result = dict(
            name=self.name,
            existing=True
        )

        if self.delete_url:
            result['delete_url'] = self.delete_url

        if self.view_url:
            result['view_url'] = self.view_url

        return result



import json

from django.core.serializers.json import DjangoJSONEncoder

from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseNotAllowed

from django_file_form.ajaxuploader.backends.local import LocalUploadBackend
from django_file_form.ajaxuploader.signals import file_uploaded


class AjaxFileUploader(object):
    def __init__(self, backend=None, **kwargs):
        if backend is None:
            backend = LocalUploadBackend
        self.get_backend = lambda: backend(**kwargs)

    def __call__(self, request, *args, **kwargs):
        return self._ajax_upload(request, *args, **kwargs)

    def _ajax_upload(self, request, *args, **kwargs):
        if request.method == "POST":
            upload = request.FILES.get('qqfile')
            if not upload:
                return HttpResponseBadRequest("AJAX request not valid")

            filename = upload.name

            file_id = request.POST['qquuid']

            backend = self.get_backend()

            # custom filename handler
            filename = (backend.update_filename(request, filename, *args, **kwargs)
                        or filename)
            # save the file
            backend.setup(filename, *args, **kwargs)
            success = backend.upload(upload, filename, False, *args, **kwargs)

            if success:
                file_uploaded.send(sender=self.__class__, backend=backend, request=request)

            # callback
            extra_context = backend.upload_complete(request, filename, file_id, *args, **kwargs)

            # let Ajax Upload know whether we saved it or not
            ret_json = {'success': success, 'filename': filename}
            if extra_context is not None:
                ret_json.update(extra_context)

            # although "application/json" is the correct content type, IE throws a fit
            return HttpResponse(json.dumps(ret_json, cls=DjangoJSONEncoder), content_type='text/html; charset=utf-8')
        else:
            response = HttpResponseNotAllowed(['POST'])
            response.write("ERROR: Only POST allowed")
            return response



from django.contrib import admin

from . import models


class UploadedFileAdmin(admin.ModelAdmin):
    list_display = ['original_filename', 'created']
    date_hierarchy = 'created'


admin.site.register(models.UploadedFile, UploadedFileAdmin)



import uuid

from django.core.exceptions import PermissionDenied

from .ajaxuploader.backends.local import LocalUploadBackend
from .ajaxuploader.views import AjaxFileUploader

from .models import UploadedFile
from . import conf


class FileFormUploadBackend(LocalUploadBackend):
    UPLOAD_DIR = 'temp_uploads'

    def upload_complete(self, request, filename, file_id, *args, **kwargs):
        result = super(FileFormUploadBackend, self).upload_complete(request, filename, file_id, *args, **kwargs)

        # get the filename if only resized image is uploaded
        if request.POST.get('qqfilename'):
            original_filename = request.POST['qqfilename']
        else:
            original_filename = request.FILES['qqfile'].name

        values = dict(
            uploaded_file='{0!s}/{1!s}'.format(self.UPLOAD_DIR, filename),
            file_id=file_id,
            form_id=request.POST['form_id'],
            original_filename=original_filename,
        )

        field_name = request.POST.get('field_name')
        if field_name:
            values['field_name'] = field_name

        UploadedFile.objects.create(**values)

        return result

    def update_filename(self, request, filename, *args, **kwargs):
        return uuid.uuid4().hex


class FileFormUploader(AjaxFileUploader):
    def __init__(self, backend=None, **kwargs):
        backend = backend or FileFormUploadBackend

        super(FileFormUploader, self).__init__(backend, **kwargs)

    def __call__(self, request, *args, **kwargs):
        if conf.MUST_LOGIN and not request.user.is_authenticated():
            raise PermissionDenied()

        return super(FileFormUploader, self).__call__(request, *args, **kwargs)
